library(readstata13)
library(dplyr, warn.conflicts = FALSE)
library(RPostgreSQL)
library(googledrive)

pg <- dbConnect(PostgreSQL())

# gs_auth()?
# bcgw_2018_ng_data_2003to2016.dta
# https://drive.google.com/open?id=112gbVzZch_ASG0ZUL62C2i2Lvu__ZUn7
data_file <- drive_download(file = as_id("112gbVzZch_ASG0ZUL62C2i2Lvu__ZUn7"), overwrite = TRUE) # Uses file id from googledrive URL
bcgw <- read.dta13(data_file$local_path)

dbGetQuery(pg, "DROP TABLE IF EXISTS non_gaap.bcgw")

bcgw<-
    bcgw %>% 
    as.data.frame() %>% 
    rename(mgr_ng_eps = MGR_NG_EPS) %>% 
    copy_to(pg, ., name = 'bcgw', temporary = FALSE)

dbGetQuery(pg, "ALTER TABLE public.bcgw SET SCHEMA non_gaap")
dbGetQuery(pg, "ALTER TABLE non_gaap.bcgw OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON non_gaap.bcgw TO non_gaap_access")

db_comment <- paste0("CREATED USING create_bcgw.R ON ", Sys.time())
dbGetQuery(pg, sprintf("COMMENT ON TABLE non_gaap.bcgw IS '%s';", db_comment))

dbDisconnect(pg)
