library(RPostgreSQL)
library(dplyr)

# Local machine
pg <- dbConnect(PostgreSQL())
rs <- dbGetQuery(pg, "SET search_path TO non_gaap")

# WARNING: set wrds_user and wrds_password before running this code
pg_wrds <- src_postgres(host = "wrds-pgdata.wharton.upenn.edu", port = 9737,
                        user = wrds_user, password = wrds_password, dbname = "wrds")
phqus <- tbl(pg_wrds, sql(paste0("SELECT * FROM comph.phqus"))) # Compustat Preliminary History - Quarterly, US

# DATADATE	DATE	Data Date
# RDQ	DATE	Report Date of Qtrly Earnings
# EPSFIQ_P - EPS (Dil.) - Including Extd'y Items (Q7)---PRELIMINARY
# SRCQ	NUM	Source Document

# SRCQ codes:
# 2	Wall Street Journal
# 8	Form 8-10-Q
# 20	Newswire
# 30	News Release
# 33	Quarterly Report + News Release
# 35	10-Q + News Release
# 38	Quarterly Report + 10-Q + News Release

dbGetQuery(pg, "DROP TABLE IF EXISTS prelim")

# The preliminary variables have suffix _P in their names, 
# while final variables do not have any suffix.
prelim <- 
    phqus %>%
    select(gvkey, datadate, rdq, epsfiq_p, srcq) %>%
    copy_to(pg, ., name = "prelim", temporary = FALSE,
            indexes=list(c("gvkey", "datadate"), c("gvkey")))

dbGetQuery(pg, "ALTER TABLE prelim OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON TABLE prelim TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE prelim")

comment <- 'CREATED USING create_prelim.R  from GitHub azakolyukina/non_gaap'
db_comment <- paste0("COMMENT ON TABLE prelim IS '",
                     comment, " ON ", Sys.time() , "'; ") 
dbGetQuery(pg, db_comment)

dbDisconnect(pg)

