# This is aux. code that evaluates of the objective function once

# Load packages, initialize within-box parallelization, set path

# Load packages
# using Pkg
# Pkg.add("PACKAGE_NAME")
using ParallelDataTransfer, Distributed
using Statistics, LinearAlgebra, Plots, Distributions, StatsBase
using DataFrames, GLM, CSV, DelimitedFiles
using SharedArrays, Random, Printf, Dates
using Roots # Roots for steady-state q

# Set path and the number of cores
if Sys.islinux() || Sys.iswindows()

    host_str = read(pipeline(`hostname`), String)
    host = replace(host_str, "\n" => "")

end

addprocs()

#-------------------------------------------------------------------------------------------------------#
# Load function files

# WARNING: set working directory to /mz_share/model here
path = "/home/aaz/research/_share/mz_share/model"
cd(path)

include("base_functions.jl") # Includes all non-primary functions
include("model.jl") # Computes model for state space
include("sim_model.jl") # Simulates model
include("calc_moments.jl") # Computes moments from simulated data
include("obj_func.jl") # Computes the squared-distance between data and simulated moments
include("def_settings.jl") # Define settings for the model

#-------------------------------------------------------------------------------------------------------#
# Evaluate objective function

# Baseline estimates from Talbe 4
x = [0.5745409793455529
     0.3381737012722537
     0.4884917981470520
     0.1267994934858418
     0.0212190418166888
     0.3859639932137643
     28.5047986163777587
     0.0110926793255593]

@time begin
obj = obj_func(x, lb_ub,
               data_moments, W, delta, r_inv, coeff_damp, min_pts,
               q_num, nu_y_num, m_y, nu_pi_num, m_pi,
               exog_num, w_num, b_num,
               policy_tol, price_tol,
               howard_iter, max_iter, price_iter,
               obs, t, sims, tburn, qseed, exog_uniform, pshuffle, shuffleshocks,
               bunchtol, max_sim_bunch, wins,
               mom_index, mom_labels, mom_index_pos, mom_index_neg, price_coef_init, fixed_coef, psi_q_lb, q_space,
               is_incl_psi = true, is_incl_b = is_incl_b, is_incl_w = is_incl_w,
               is_price_fixed = is_price_fixed, fixed_beta = price_coef[1:end],
               is_x_grid_x = is_x_grid_x, write_txt = true, txt_name = string("/Output/moments.txt"),
               suffix = suffix, prog_file = missing)
end
