function obj_func(x, # 8x1 alpha, kappa_w, rho_y, sigma_y, sigma_pi, theta, kappa_b, mu_pi
                  lb_ub, # px2 element vector for param. bounds
                  data_moments,
                  W, # moment covariance matrix
                  delta, r_inv, coeff_damp, min_pts,
                  q_num, nu_y_num, m_y, nu_pi_num, m_pi,
                  exog_num, w_num, b_num,
                  policy_tol, price_tol,
                  howard_iter, max_iter, price_iter,
                  obs, t, sims, tburn, qseed, exog_uniform, pshuffle, shuffleshocks,
                  bunchtol, max_sim_bunch, wins,
                  mom_index, mom_labels, mom_index_pos, mom_index_neg, price_coef_init, fixed_coef, psi_q_lb, q_space;
                  is_incl_psi = true, is_incl_b = true, is_incl_w = is_incl_w,
                  is_price_fixed = false, fixed_beta = missing, is_comp_static = is_comp_static,
                  is_x_grid_x = true, write_txt = false, txt_name = "moments.txt", suffix = "bea", prog_file=prog_file)
    # This funciton computes squared distance between simulated moments and data moments

    # Initialize objective func
    obj = nothing
    fvalmax = 100000.0

    # Initialize flags
    price_flag = 99
    rho_flag = 99
    sing_flag = 99
    flag_coef = 99
    vf_flag = 99
    qmid_flag = 99
    bunch_flag = fill(99, 5)
    moments = Array{Float64}(undef, length(mom_index))

    # Check that parameter values are within bounds
    # WARNING: Do element-wise comparison---this condition is sensitive
    # to param as vec() or param as array()---to allow for both
    if (# Any below bounds?
       lb_ub[1, 1] - 1e-8 >= x[1] || lb_ub[2, 1] - 1e-8 >= x[2] ||
       lb_ub[3, 1] - 1e-8 >= x[3] || lb_ub[4, 1] - 1e-8 >= x[4] ||
       lb_ub[5, 1] - 1e-8 >= x[5] || lb_ub[6, 1] - 1e-8 >= x[6] ||
       lb_ub[7, 1] - 1e-8 >= x[7] || lb_ub[8, 1] - 1e-8 >= x[8] ||
       # Any above bounds?
       lb_ub[1, 2] + 1e-8 <= x[1] || lb_ub[2, 2] + 1e-8 <= x[2] ||
       lb_ub[3, 2] + 1e-8 <= x[3] || lb_ub[4, 2] + 1e-8 <= x[4] ||
       lb_ub[5, 2] + 1e-8 <= x[5] || lb_ub[6, 2] + 1e-8 <= x[6] ||
       lb_ub[7, 2] + 1e-8 <= x[7] || lb_ub[8, 2] + 1e-8 <= x[8])

        obj = fvalmax # set objective to be very high
        beta = price_coef_init
        println("Parameter step out of bounds")

    else

        if write_txt

            bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag,
             qmid_flag, moments, beta =
                calc_moments(x,
                             delta, r_inv, coeff_damp, min_pts,
                             q_num, nu_y_num, m_y, nu_pi_num, m_pi,
                             exog_num, w_num, b_num,
                             policy_tol, price_tol, pshuffle,
                             howard_iter, max_iter, price_iter,
                             obs, t, tburn, sims, qseed, exog_uniform, shuffleshocks,
                             bunchtol, max_sim_bunch, wins,
                             mom_index, mom_labels, price_coef_init, fixed_coef, psi_q_lb, q_space,
                             is_incl_psi = is_incl_psi, is_incl_b = is_incl_b, is_incl_w = is_incl_w,
                             is_price_fixed = is_price_fixed, fixed_beta = fixed_beta,
                             return_prices = true, is_comp_static = is_comp_static,
                             is_x_grid_x = is_x_grid_x)

        else

            bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments =
                calc_moments(x,
                             delta, r_inv, coeff_damp, min_pts,
                             q_num, nu_y_num, m_y, nu_pi_num, m_pi,
                             exog_num, w_num, b_num,
                             policy_tol, price_tol, pshuffle,
                             howard_iter, max_iter, price_iter,
                             obs, t, tburn, sims, qseed, exog_uniform, shuffleshocks,
                             bunchtol, max_sim_bunch, wins,
                             mom_index, mom_labels, price_coef_init, fixed_coef, psi_q_lb, q_space,
                             is_incl_psi = is_incl_psi, is_incl_b = is_incl_b, is_incl_w = is_incl_w,
                             is_price_fixed = is_price_fixed, fixed_beta = fixed_beta,
                             is_comp_static = is_comp_static, is_x_grid_x = is_x_grid_x)

        end

        # Set objective to fvalmax if one of the flags gets triggered
        if maximum(bunch_flag) >= 1
            obj = fvalmax
        elseif price_flag == 1
            obj = fvalmax
        elseif rho_flag == 1
            obj = fvalmax
        elseif sing_flag == 1
            obj = fvalmax
        elseif flag_coef == 1
            obj = fvalmax
        elseif vf_flag == 1
            obj = fvalmax
        elseif qmid_flag == 1
            obj = fvalmax
        else
            if all((isempty(mom_index_pos) || all(moments[mom_index_pos] .> fill(1e-8, length(mom_index_pos))))
                && (isempty(mom_index_neg) || all(moments[mom_index_neg] .< fill(-1e-8, length(mom_index_neg)))))
                obj = (data_moments - moments)'*W*(data_moments - moments)
                obj = obj[1]
            else
                obj = fvalmax
            end
        end
    end

    if write_txt

        # Set column and row names for output
        len_max = 12 # max length for text string
        param_names = ["alpha", "kappa_w", "rho_y", "sigma_y", "sigma_pi", "theta", "kappa_b", "mu_pi"] # parameters
        param_names_print = param_names
        for i in 1:length(param_names)
            param_names_print[i] = string(param_names[i], repeat(" ", len_max - length(param_names[i])))
        end

        len_max = 30 # max length for text string
        price_names = ["beta_0", "beta_1", "beta_2"]; # price names
        if is_incl_w
            price_names = [price_names; "beta3"] # append w beta to pricing
        end


        price_names_print = price_names
        for i in 1:length(price_names)
            price_names_print[i] = string(price_names[i], repeat(" ", len_max - length(price_names[i])))
        end

        # Add tabs to moment labels so the txt output aligns
        len_max = 35 # max length for text string
        mom_labels_print = mom_labels[mom_index]
        for i in 1:length(mom_labels_print)
            mom_labels_print[i] = string(mom_labels_print[i], repeat(" ", len_max - length(mom_labels_print[i])))
        end

        prices     = beta # RE prices

        open(string(path, txt_name), "w") do f

            write(f, @sprintf "                   Moments calculated on %s\n\n" Dates.format(now(), "yyyy-mm-dd HH:MM:S"))

            # Print setup
            write(f, @sprintf "intangibles type:              %s\n" suffix)
            write(f, @sprintf "include bias (model):          %s\n" is_incl_b)
            write(f, @sprintf "include w in (model):          %s\n" is_incl_w)
            write(f, @sprintf "price is fixed (pricing):      %s\n" is_price_fixed)

            # Write table on estimated parameter values
            write(f, "\n Parameter values\n\n")
            for mi in 1:length(param_names_print)
                write(f, @sprintf "  %s%.16f %s\n" spaces(x[mi]) x[mi] param_names_print[mi])
            end

            # Write table on pricing parameters
            write(f, "\n Pricing parameters\n\n")
            for mi in 1:length(price_names)
                write(f, @sprintf "  %s%.16f %s\n" spaces(prices[mi]) prices[mi] price_names_print[mi])
            end

            # Report GMM value
            write(f, @sprintf "\nCurrent GMM value        %.5f\n\n" obj)

            # Write table on simulated and data moments
            write(f, "    Moments                             Simulated      Data\n\n")
            for mi in 1:length(mom_labels_print)
                write(f, @sprintf "%2i  %s%s%.8f    %s%.8f\n" mi mom_labels_print[mi] spaces(moments[mi]) moments[mi] spaces(data_moments[mi]) data_moments[mi])
            end
            write(f, "\n")

        end

    end

    println(@sprintf("x: %2.16f %2.16f %2.16f %2.16f %2.16f %2.16f %2.16f %2.16f",
                     x[1], x[2], x[3], x[4], x[5], x[6], x[7], x[8]))

    # Print flags
    strpad(x) = repeat(" ", max(0, 9 - length(string(x))))
    println(@sprintf("Flags header %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s",
                     "prc", strpad("prc"), "rho", strpad("rho"), "sing", strpad("sing"), "coef", strpad("coef"),
                     "vf", strpad("vf"), "qmid", strpad("qmid"), "qu", strpad("qu"), "ql", strpad("ql"),
                     "wu", strpad("wu"), "bu", strpad("bu"), "bl", strpad("bl")))
    println(@sprintf("Flags        %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s %s%s",
                     string(price_flag), strpad(price_flag), string(rho_flag), strpad(rho_flag),
                     string(sing_flag), strpad(sing_flag), string(flag_coef), strpad(flag_coef),
                     string(vf_flag), strpad(vf_flag), string(qmid_flag), strpad(qmid_flag),
                     string(bunch_flag[1]), strpad(bunch_flag[1]), string(bunch_flag[2]), strpad(bunch_flag[2]),
                     string(bunch_flag[3]), strpad(bunch_flag[3]), string(bunch_flag[4]), strpad(bunch_flag[4]),
                     string(bunch_flag[5]), strpad(bunch_flag[5])))
    println("F(x)         = ", obj, "\n")

    # Squared distance between simulated and actual moments
    return obj[1]
end
